// The MIT License
//
// Copyright (c) 2018 Google, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

import React from 'react';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import withRipple from 'closure-react-ripple/index';

export const Button = (props) => {
  const {
    className,
    raised,
    unelevated,
    outlined,
    dense,
    icon,
    children,
    initRipple,
    unbounded, // eslint-disable-line no-unused-vars
    ...otherProps
  } = props;

  const classes = classnames('mdc-button', className, {
    'mdc-button--raised': raised,
    'mdc-button--unelevated': unelevated,
    'mdc-button--outlined': outlined,
    'mdc-button--dense': dense,
  });

  const SemanticButton = props.href ? 'a' : 'button';

  return (
    <SemanticButton
      className={classes}
      ref={initRipple}
      {...otherProps}
    >
      {icon ? renderIcon(icon) : null}
      {children}
    </SemanticButton>
  );
};

const addClassesToElement = (classes, element) => {
  const propsWithClasses = {
    className: classnames(classes, element.props.className),
  };
  return React.cloneElement(element, propsWithClasses);
};

const renderIcon = (icon) => addClassesToElement('mdc-button__icon', icon);

Button.propTypes = {
  raised: PropTypes.bool,
  unelevated: PropTypes.bool,
  outlined: PropTypes.bool,
  dense: PropTypes.bool,
  disabled: PropTypes.bool,
  unbounded: PropTypes.bool,
  initRipple: PropTypes.func,
  className: PropTypes.string,
  icon: PropTypes.element,
  href: PropTypes.string,
  children: PropTypes.string,
};

/**
 * @nocollapse
 */
Button.defaultProps = {
  raised: false,
  unelevated: false,
  outlined: false,
  dense: false,
  disabled: false,
  unbounded: false,
  initRipple: () => {},
  className: '',
  icon: null,
  children: '',
};

export default withRipple(Button);
